package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMASSESSMX;
import cn.ibizlab.eam.core.eam_core.filter.EMASSESSMXSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMASSESSMXService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMASSESSMXMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[设备停用明细考核] 服务对象接口实现
 */
@Slf4j
@Service("EMASSESSMXServiceImpl")
public class EMASSESSMXServiceImpl extends ServiceImpl<EMASSESSMXMapper, EMASSESSMX> implements IEMASSESSMXService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMDISABLEASSESSService emdisableassessService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQUIPService emequipService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean create(EMASSESSMX et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmassessmxid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMASSESSMX> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    public EMASSESSMX getDraft(EMASSESSMX et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public EMASSESSMX get(String key) {
        EMASSESSMX et = getById(key);
        if(et==null){
            et=new EMASSESSMX();
            et.setEmassessmxid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public boolean checkKey(EMASSESSMX et) {
        return (!ObjectUtils.isEmpty(et.getEmassessmxid()))&&(!Objects.isNull(this.getById(et.getEmassessmxid())));
    }

    @Override
    @Transactional
    public boolean update(EMASSESSMX et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emassessmxid",et.getEmassessmxid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmassessmxid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMASSESSMX> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMASSESSMX et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMASSESSMX et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMASSESSMX> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EMASSESSMX> selectByEmdisableassessid(String emdisableassessid) {
        return baseMapper.selectByEmdisableassessid(emdisableassessid);
    }

    @Override
    public void removeByEmdisableassessid(String emdisableassessid) {
        this.remove(new QueryWrapper<EMASSESSMX>().eq("emdisableassessid",emdisableassessid));
    }

	@Override
    public List<EMASSESSMX> selectByEmequipid(String emequipid) {
        return baseMapper.selectByEmequipid(emequipid);
    }

    @Override
    public void removeByEmequipid(String emequipid) {
        this.remove(new QueryWrapper<EMASSESSMX>().eq("emequipid",emequipid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMASSESSMX> searchDefault(EMASSESSMXSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMASSESSMX> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMASSESSMX>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMASSESSMX et){
        //实体关系[DER1N_EMASSESSMX_EMDISABLEASSESS_EMDISABLEASSESSID]
        if(!ObjectUtils.isEmpty(et.getEmdisableassessid())){
            cn.ibizlab.eam.core.eam_core.domain.EMDISABLEASSESS emdisableassess=et.getEmdisableassess();
            if(ObjectUtils.isEmpty(emdisableassess)){
                cn.ibizlab.eam.core.eam_core.domain.EMDISABLEASSESS majorEntity=emdisableassessService.get(et.getEmdisableassessid());
                et.setEmdisableassess(majorEntity);
                emdisableassess=majorEntity;
            }
            et.setEmdisableassessname(emdisableassess.getEmdisableassessname());
        }
        //实体关系[DER1N_EMASSESSMX_EMEQUIP_EMEQUIPID]
        if(!ObjectUtils.isEmpty(et.getEmequipid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEQUIP emequip=et.getEmequip();
            if(ObjectUtils.isEmpty(emequip)){
                cn.ibizlab.eam.core.eam_core.domain.EMEQUIP majorEntity=emequipService.get(et.getEmequipid());
                et.setEmequip(majorEntity);
                emequip=majorEntity;
            }
            et.setEmequipname(emequip.getEquipinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


