package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMASSETHIST;
import cn.ibizlab.eam.core.eam_core.filter.EMASSETHISTSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMASSETHISTService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMASSETHISTMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[资产历史记录] 服务对象接口实现
 */
@Slf4j
@Service("EMASSETHISTServiceImpl")
public class EMASSETHISTServiceImpl extends ServiceImpl<EMASSETHISTMapper, EMASSETHIST> implements IEMASSETHISTService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMASSETService emassetService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(EMASSETHIST et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emassethistid",et.getEmassethistid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmassethistid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMASSETHIST> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean save(EMASSETHIST et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMASSETHIST et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMASSETHIST> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(EMASSETHIST et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmassethistid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMASSETHIST> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(EMASSETHIST et) {
        return (!ObjectUtils.isEmpty(et.getEmassethistid()))&&(!Objects.isNull(this.getById(et.getEmassethistid())));
    }

    @Override
    public EMASSETHIST getDraft(EMASSETHIST et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public EMASSETHIST get(String key) {
        EMASSETHIST et = getById(key);
        if(et==null){
            et=new EMASSETHIST();
            et.setEmassethistid(key);
        }
        else{
        }
        return et;
    }


	@Override
    public List<EMASSETHIST> selectByAssetid(String emassetid) {
        return baseMapper.selectByAssetid(emassetid);
    }

    @Override
    public void removeByAssetid(String emassetid) {
        this.remove(new QueryWrapper<EMASSETHIST>().eq("assetid",emassetid));
    }

	@Override
    public List<EMASSETHIST> selectByEmpid(String pfempid) {
        return baseMapper.selectByEmpid(pfempid);
    }

    @Override
    public void removeByEmpid(String pfempid) {
        this.remove(new QueryWrapper<EMASSETHIST>().eq("empid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMASSETHIST> searchDefault(EMASSETHISTSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMASSETHIST> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMASSETHIST>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMASSETHIST et){
        //实体关系[DER1N_EMASSETHIST_EMASSET_ASSETID]
        if(!ObjectUtils.isEmpty(et.getAssetid())){
            cn.ibizlab.eam.core.eam_core.domain.EMASSET asset=et.getAsset();
            if(ObjectUtils.isEmpty(asset)){
                cn.ibizlab.eam.core.eam_core.domain.EMASSET majorEntity=emassetService.get(et.getAssetid());
                et.setAsset(majorEntity);
                asset=majorEntity;
            }
            et.setAssetname(asset.getAssetinfo());
        }
        //实体关系[DER1N_EMASSETHIST_PFEMP_EMPID]
        if(!ObjectUtils.isEmpty(et.getEmpid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP emp=et.getEmp();
            if(ObjectUtils.isEmpty(emp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getEmpid());
                et.setEmp(majorEntity);
                emp=majorEntity;
            }
            et.setEmpname(emp.getEmpinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


