package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMASSET;
import cn.ibizlab.eam.core.eam_core.filter.EMASSETSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMASSETService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMASSETMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[资产] 服务对象接口实现
 */
@Slf4j
@Service("EMASSETServiceImpl")
public class EMASSETServiceImpl extends ServiceImpl<EMASSETMapper, EMASSET> implements IEMASSETService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMASSETCLEARService emassetclearService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMASSETHISTService emassethistService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQUIPService emequipService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMACCLASSService emacclassService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMASSETCLASSService emassetclassService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQLOCATIONService emeqlocationService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSERVICEService emserviceService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFCONTRACTService pfcontractService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFDEPTService pfdeptService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFUNITService pfunitService;

    private int batchSize = 500;

    @Override
    @Transactional
    public EMASSET get(String key) {
        EMASSET et = getById(key);
        if(et==null){
            et=new EMASSET();
            et.setEmassetid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean save(EMASSET et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMASSET et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMASSET> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EMASSET et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emassetid",et.getEmassetid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmassetid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMASSET> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(EMASSET et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmassetid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMASSET> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public EMASSET getDraft(EMASSET et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EMASSET et) {
        return (!ObjectUtils.isEmpty(et.getEmassetid()))&&(!Objects.isNull(this.getById(et.getEmassetid())));
    }


	@Override
    public List<EMASSET> selectByAcclassid(String emacclassid) {
        return baseMapper.selectByAcclassid(emacclassid);
    }

    @Override
    public void removeByAcclassid(String emacclassid) {
        this.remove(new QueryWrapper<EMASSET>().eq("acclassid",emacclassid));
    }

	@Override
    public List<EMASSET> selectByAssetclassid(String emassetclassid) {
        return baseMapper.selectByAssetclassid(emassetclassid);
    }

    @Override
    public void removeByAssetclassid(String emassetclassid) {
        this.remove(new QueryWrapper<EMASSET>().eq("assetclassid",emassetclassid));
    }

	@Override
    public List<EMASSET> selectByEqlocationid(String emeqlocationid) {
        return baseMapper.selectByEqlocationid(emeqlocationid);
    }

    @Override
    public void removeByEqlocationid(String emeqlocationid) {
        this.remove(new QueryWrapper<EMASSET>().eq("eqlocationid",emeqlocationid));
    }

	@Override
    public List<EMASSET> selectByLabserviceid(String emserviceid) {
        return baseMapper.selectByLabserviceid(emserviceid);
    }

    @Override
    public void removeByLabserviceid(String emserviceid) {
        this.remove(new QueryWrapper<EMASSET>().eq("labserviceid",emserviceid));
    }

	@Override
    public List<EMASSET> selectByMserviceid(String emserviceid) {
        return baseMapper.selectByMserviceid(emserviceid);
    }

    @Override
    public void removeByMserviceid(String emserviceid) {
        this.remove(new QueryWrapper<EMASSET>().eq("mserviceid",emserviceid));
    }

	@Override
    public List<EMASSET> selectByRserviceid(String emserviceid) {
        return baseMapper.selectByRserviceid(emserviceid);
    }

    @Override
    public void removeByRserviceid(String emserviceid) {
        this.remove(new QueryWrapper<EMASSET>().eq("rserviceid",emserviceid));
    }

	@Override
    public List<EMASSET> selectByContractid(String pfcontractid) {
        return baseMapper.selectByContractid(pfcontractid);
    }

    @Override
    public void removeByContractid(String pfcontractid) {
        this.remove(new QueryWrapper<EMASSET>().eq("contractid",pfcontractid));
    }

	@Override
    public List<EMASSET> selectByDeptid(String pfdeptid) {
        return baseMapper.selectByDeptid(pfdeptid);
    }

    @Override
    public void removeByDeptid(String pfdeptid) {
        this.remove(new QueryWrapper<EMASSET>().eq("deptid",pfdeptid));
    }

	@Override
    public List<EMASSET> selectByMgrdeptid(String pfdeptid) {
        return baseMapper.selectByMgrdeptid(pfdeptid);
    }

    @Override
    public void removeByMgrdeptid(String pfdeptid) {
        this.remove(new QueryWrapper<EMASSET>().eq("mgrdeptid",pfdeptid));
    }

	@Override
    public List<EMASSET> selectByEmpid(String pfempid) {
        return baseMapper.selectByEmpid(pfempid);
    }

    @Override
    public void removeByEmpid(String pfempid) {
        this.remove(new QueryWrapper<EMASSET>().eq("empid",pfempid));
    }

	@Override
    public List<EMASSET> selectByRempid(String pfempid) {
        return baseMapper.selectByRempid(pfempid);
    }

    @Override
    public void removeByRempid(String pfempid) {
        this.remove(new QueryWrapper<EMASSET>().eq("rempid",pfempid));
    }

	@Override
    public List<EMASSET> selectByUnitid(String pfunitid) {
        return baseMapper.selectByUnitid(pfunitid);
    }

    @Override
    public void removeByUnitid(String pfunitid) {
        this.remove(new QueryWrapper<EMASSET>().eq("unitid",pfunitid));
    }


    /**
     * 查询集合 报废资产
     */
    @Override
    public Page<EMASSET> searchAssetBf(EMASSETSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMASSET> pages=baseMapper.searchAssetBf(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMASSET>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMASSET> searchDefault(EMASSETSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMASSET> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMASSET>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMASSET et){
        //实体关系[DER1N_EMASSET_EMACCLASS_ACCLASSID]
        if(!ObjectUtils.isEmpty(et.getAcclassid())){
            cn.ibizlab.eam.core.eam_core.domain.EMACCLASS acclass=et.getAcclass();
            if(ObjectUtils.isEmpty(acclass)){
                cn.ibizlab.eam.core.eam_core.domain.EMACCLASS majorEntity=emacclassService.get(et.getAcclassid());
                et.setAcclass(majorEntity);
                acclass=majorEntity;
            }
            et.setAcclassname(acclass.getEmacclassname());
        }
        //实体关系[DER1N_EMASSET_EMASSETCLASS_ASSETCLASSID]
        if(!ObjectUtils.isEmpty(et.getAssetclassid())){
            cn.ibizlab.eam.core.eam_core.domain.EMASSETCLASS assetclass=et.getAssetclass();
            if(ObjectUtils.isEmpty(assetclass)){
                cn.ibizlab.eam.core.eam_core.domain.EMASSETCLASS majorEntity=emassetclassService.get(et.getAssetclassid());
                et.setAssetclass(majorEntity);
                assetclass=majorEntity;
            }
            et.setAssetclassname(assetclass.getAssetclassinfo());
            et.setAssetclasscode(assetclass.getAssetclasscode());
        }
        //实体关系[DER1N_EMASSET_EMEQLOCATION_EQLOCATIONID]
        if(!ObjectUtils.isEmpty(et.getEqlocationid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEQLOCATION eqlocation=et.getEqlocation();
            if(ObjectUtils.isEmpty(eqlocation)){
                cn.ibizlab.eam.core.eam_core.domain.EMEQLOCATION majorEntity=emeqlocationService.get(et.getEqlocationid());
                et.setEqlocation(majorEntity);
                eqlocation=majorEntity;
            }
            et.setEqlocationname(eqlocation.getEqlocationinfo());
        }
        //实体关系[DER1N_EMASSET_EMSERVICE_LABSERVICEID]
        if(!ObjectUtils.isEmpty(et.getLabserviceid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSERVICE labservice=et.getLabservice();
            if(ObjectUtils.isEmpty(labservice)){
                cn.ibizlab.eam.core.eam_core.domain.EMSERVICE majorEntity=emserviceService.get(et.getLabserviceid());
                et.setLabservice(majorEntity);
                labservice=majorEntity;
            }
            et.setLabservicename(labservice.getServiceinfo());
        }
        //实体关系[DER1N_EMASSET_EMSERVICE_MSERVICEID]
        if(!ObjectUtils.isEmpty(et.getMserviceid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSERVICE mservice=et.getMservice();
            if(ObjectUtils.isEmpty(mservice)){
                cn.ibizlab.eam.core.eam_core.domain.EMSERVICE majorEntity=emserviceService.get(et.getMserviceid());
                et.setMservice(majorEntity);
                mservice=majorEntity;
            }
            et.setMservicename(mservice.getServiceinfo());
        }
        //实体关系[DER1N_EMASSET_EMSERVICE_RSERVICEID]
        if(!ObjectUtils.isEmpty(et.getRserviceid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSERVICE rservice=et.getRservice();
            if(ObjectUtils.isEmpty(rservice)){
                cn.ibizlab.eam.core.eam_core.domain.EMSERVICE majorEntity=emserviceService.get(et.getRserviceid());
                et.setRservice(majorEntity);
                rservice=majorEntity;
            }
            et.setRservicename(rservice.getServiceinfo());
        }
        //实体关系[DER1N_EMASSET_PFCONTRACT_CONTRACTID]
        if(!ObjectUtils.isEmpty(et.getContractid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFCONTRACT contract=et.getContract();
            if(ObjectUtils.isEmpty(contract)){
                cn.ibizlab.eam.core.eam_pf.domain.PFCONTRACT majorEntity=pfcontractService.get(et.getContractid());
                et.setContract(majorEntity);
                contract=majorEntity;
            }
            et.setContractname(contract.getContractinfo());
        }
        //实体关系[DER1N_EMASSET_PFDEPT_DEPTID]
        if(!ObjectUtils.isEmpty(et.getDeptid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFDEPT dept=et.getDept();
            if(ObjectUtils.isEmpty(dept)){
                cn.ibizlab.eam.core.eam_pf.domain.PFDEPT majorEntity=pfdeptService.get(et.getDeptid());
                et.setDept(majorEntity);
                dept=majorEntity;
            }
            et.setDeptname(dept.getDeptinfo());
            et.setMgrempname(dept.getMgrempname());
        }
        //实体关系[DER1N_EMASSET_PFDEPT_MGRDEPTID]
        if(!ObjectUtils.isEmpty(et.getMgrdeptid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFDEPT mgrdept=et.getMgrdept();
            if(ObjectUtils.isEmpty(mgrdept)){
                cn.ibizlab.eam.core.eam_pf.domain.PFDEPT majorEntity=pfdeptService.get(et.getMgrdeptid());
                et.setMgrdept(majorEntity);
                mgrdept=majorEntity;
            }
            et.setMgrdeptmgr(mgrdept.getMgrempname());
            et.setMgrdeptname(mgrdept.getDeptinfo());
        }
        //实体关系[DER1N_EMASSET_PFEMP_EMPID]
        if(!ObjectUtils.isEmpty(et.getEmpid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP emp=et.getEmp();
            if(ObjectUtils.isEmpty(emp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getEmpid());
                et.setEmp(majorEntity);
                emp=majorEntity;
            }
            et.setEmpname(emp.getEmpinfo());
        }
        //实体关系[DER1N_EMASSET_PFEMP_REMPID]
        if(!ObjectUtils.isEmpty(et.getRempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP remp=et.getRemp();
            if(ObjectUtils.isEmpty(remp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getRempid());
                et.setRemp(majorEntity);
                remp=majorEntity;
            }
            et.setRempname(remp.getEmpinfo());
        }
        //实体关系[DER1N_EMASSET_PFUNIT_UNITID]
        if(!ObjectUtils.isEmpty(et.getUnitid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFUNIT unit=et.getUnit();
            if(ObjectUtils.isEmpty(unit)){
                cn.ibizlab.eam.core.eam_pf.domain.PFUNIT majorEntity=pfunitService.get(et.getUnitid());
                et.setUnit(majorEntity);
                unit=majorEntity;
            }
            et.setUnitname(unit.getPfunitname());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


