package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMCAB;
import cn.ibizlab.eam.core.eam_core.filter.EMCABSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMCABService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMCABMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[货架] 服务对象接口实现
 */
@Slf4j
@Service("EMCABServiceImpl")
public class EMCABServiceImpl extends ServiceImpl<EMCABMapper, EMCAB> implements IEMCABService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMService emitemService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSTOREPARTService emstorepartService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSTOREService emstoreService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EMCAB et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmcabid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMCAB> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(EMCAB et) {
        return (!ObjectUtils.isEmpty(et.getEmcabid()))&&(!Objects.isNull(this.getById(et.getEmcabid())));
    }

    @Override
    public EMCAB getDraft(EMCAB et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean save(EMCAB et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMCAB et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMCAB> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public EMCAB get(String key) {
        EMCAB et = getById(key);
        if(et==null){
            et=new EMCAB();
            et.setEmcabid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean update(EMCAB et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emcabid",et.getEmcabid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmcabid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMCAB> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }


	@Override
    public List<EMCAB> selectByEmstorepartid(String emstorepartid) {
        return baseMapper.selectByEmstorepartid(emstorepartid);
    }

    @Override
    public void removeByEmstorepartid(String emstorepartid) {
        this.remove(new QueryWrapper<EMCAB>().eq("emstorepartid",emstorepartid));
    }

	@Override
    public List<EMCAB> selectByEmstoreid(String emstoreid) {
        return baseMapper.selectByEmstoreid(emstoreid);
    }

    @Override
    public void removeByEmstoreid(String emstoreid) {
        this.remove(new QueryWrapper<EMCAB>().eq("emstoreid",emstoreid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMCAB> searchDefault(EMCABSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMCAB> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMCAB>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMCAB et){
        //实体关系[DER1N_EMCAB_EMSTOREPART_EMSTOREPARTID]
        if(!ObjectUtils.isEmpty(et.getEmstorepartid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSTOREPART emstorepart=et.getEmstorepart();
            if(ObjectUtils.isEmpty(emstorepart)){
                cn.ibizlab.eam.core.eam_core.domain.EMSTOREPART majorEntity=emstorepartService.get(et.getEmstorepartid());
                et.setEmstorepart(majorEntity);
                emstorepart=majorEntity;
            }
            et.setEmstorepartname(emstorepart.getEmstorepartname());
        }
        //实体关系[DER1N_EMCAB_EMSTORE_EMSTOREID]
        if(!ObjectUtils.isEmpty(et.getEmstoreid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSTORE emstore=et.getEmstore();
            if(ObjectUtils.isEmpty(emstore)){
                cn.ibizlab.eam.core.eam_core.domain.EMSTORE majorEntity=emstoreService.get(et.getEmstoreid());
                et.setEmstore(majorEntity);
                emstore=majorEntity;
            }
            et.setEmstorename(emstore.getStoreinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


