package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMENCONSUM;
import cn.ibizlab.eam.core.eam_core.filter.EMENCONSUMSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMENCONSUMService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMENCONSUMMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[能耗] 服务对象接口实现
 */
@Slf4j
@Service("EMENCONSUMServiceImpl")
public class EMENCONSUMServiceImpl extends ServiceImpl<EMENCONSUMMapper, EMENCONSUM> implements IEMENCONSUMService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMENService emenService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQUIPService emequipService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMOBJECTService emobjectService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMWOService emwoService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFDEPTService pfdeptService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean save(EMENCONSUM et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMENCONSUM et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMENCONSUM> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public EMENCONSUM get(String key) {
        EMENCONSUM et = getById(key);
        if(et==null){
            et=new EMENCONSUM();
            et.setEmenconsumid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean update(EMENCONSUM et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emenconsumid",et.getEmenconsumid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmenconsumid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMENCONSUM> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(EMENCONSUM et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmenconsumid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMENCONSUM> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public EMENCONSUM getDraft(EMENCONSUM et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EMENCONSUM et) {
        return (!ObjectUtils.isEmpty(et.getEmenconsumid()))&&(!Objects.isNull(this.getById(et.getEmenconsumid())));
    }


	@Override
    public List<EMENCONSUM> selectByEnid(String emenid) {
        return baseMapper.selectByEnid(emenid);
    }

    @Override
    public void removeByEnid(String emenid) {
        this.remove(new QueryWrapper<EMENCONSUM>().eq("enid",emenid));
    }

	@Override
    public List<EMENCONSUM> selectByEquipid(String emequipid) {
        return baseMapper.selectByEquipid(emequipid);
    }

    @Override
    public void removeByEquipid(String emequipid) {
        this.remove(new QueryWrapper<EMENCONSUM>().eq("equipid",emequipid));
    }

	@Override
    public List<EMENCONSUM> selectByObjid(String emobjectid) {
        return baseMapper.selectByObjid(emobjectid);
    }

    @Override
    public void removeByObjid(String emobjectid) {
        this.remove(new QueryWrapper<EMENCONSUM>().eq("objid",emobjectid));
    }

	@Override
    public List<EMENCONSUM> selectByWoid(String emwoid) {
        return baseMapper.selectByWoid(emwoid);
    }

    @Override
    public void removeByWoid(String emwoid) {
        this.remove(new QueryWrapper<EMENCONSUM>().eq("woid",emwoid));
    }

	@Override
    public List<EMENCONSUM> selectByDeptid(String pfdeptid) {
        return baseMapper.selectByDeptid(pfdeptid);
    }

    @Override
    public void removeByDeptid(String pfdeptid) {
        this.remove(new QueryWrapper<EMENCONSUM>().eq("deptid",pfdeptid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMENCONSUM> searchDefault(EMENCONSUMSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMENCONSUM> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMENCONSUM>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMENCONSUM et){
        //实体关系[DER1N_EMENCONSUM_EMEN_ENID]
        if(!ObjectUtils.isEmpty(et.getEnid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEN en=et.getEn();
            if(ObjectUtils.isEmpty(en)){
                cn.ibizlab.eam.core.eam_core.domain.EMEN majorEntity=emenService.get(et.getEnid());
                et.setEn(majorEntity);
                en=majorEntity;
            }
            et.setEnprice(en.getPrice());
            et.setItemmtypename(en.getItemmtypename());
            et.setItemname(en.getItemname());
            et.setUnitname(en.getUnitname());
            et.setItemprice(en.getItemprice());
            et.setEnname(en.getEnergyinfo());
            et.setItemtypeid(en.getItemtypeid());
            et.setItembtypeid(en.getItembtypeid());
            et.setItembtypename(en.getItembtypename());
            et.setItemid(en.getItemid());
            et.setItemmtypeid(en.getItemmtypeid());
        }
        //实体关系[DER1N_EMENCONSUM_EMEQUIP_EQUIPID]
        if(!ObjectUtils.isEmpty(et.getEquipid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEQUIP equip=et.getEquip();
            if(ObjectUtils.isEmpty(equip)){
                cn.ibizlab.eam.core.eam_core.domain.EMEQUIP majorEntity=emequipService.get(et.getEquipid());
                et.setEquip(majorEntity);
                equip=majorEntity;
            }
            et.setEquipcode(equip.getEquipcode());
            et.setEquipname(equip.getEquipinfo());
            et.setSname(equip.getSname());
        }
        //实体关系[DER1N_EMENCONSUM_EMOBJECT_OBJID]
        if(!ObjectUtils.isEmpty(et.getObjid())){
            cn.ibizlab.eam.core.eam_core.domain.EMOBJECT obj=et.getObj();
            if(ObjectUtils.isEmpty(obj)){
                cn.ibizlab.eam.core.eam_core.domain.EMOBJECT majorEntity=emobjectService.get(et.getObjid());
                et.setObj(majorEntity);
                obj=majorEntity;
            }
            et.setObjname(obj.getObjectinfo());
        }
        //实体关系[DER1N_EMENCONSUM_EMWO_WOID]
        if(!ObjectUtils.isEmpty(et.getWoid())){
            cn.ibizlab.eam.core.eam_core.domain.EMWO wo=et.getWo();
            if(ObjectUtils.isEmpty(wo)){
                cn.ibizlab.eam.core.eam_core.domain.EMWO majorEntity=emwoService.get(et.getWoid());
                et.setWo(majorEntity);
                wo=majorEntity;
            }
            et.setWoname(wo.getWoinfo());
        }
        //实体关系[DER1N_EMENCONSUM_PFDEPT_DEPTID]
        if(!ObjectUtils.isEmpty(et.getDeptid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFDEPT dept=et.getDept();
            if(ObjectUtils.isEmpty(dept)){
                cn.ibizlab.eam.core.eam_pf.domain.PFDEPT majorEntity=pfdeptService.get(et.getDeptid());
                et.setDept(majorEntity);
                dept=majorEntity;
            }
            et.setSdept(dept.getSdept());
            et.setDeptname(dept.getDeptinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


