package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMEQTYPE;
import cn.ibizlab.eam.core.eam_core.filter.EMEQTYPESearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMEQTYPEService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMEQTYPEMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[设备类型] 服务对象接口实现
 */
@Slf4j
@Service("EMEQTYPEServiceImpl")
public class EMEQTYPEServiceImpl extends ServiceImpl<EMEQTYPEMapper, EMEQTYPE> implements IEMEQTYPEService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQSTOPMONIService emeqstopmoniService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQSTOPService emeqstopService;

    private cn.ibizlab.eam.core.eam_core.service.IEMEQTYPEService emeqtypeService = this;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQUIPService emequipService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(EMEQTYPE et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emeqtypeid",et.getEmeqtypeid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmeqtypeid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMEQTYPE> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    public EMEQTYPE getDraft(EMEQTYPE et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public EMEQTYPE get(String key) {
        EMEQTYPE et = getById(key);
        if(et==null){
            et=new EMEQTYPE();
            et.setEmeqtypeid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean create(EMEQTYPE et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmeqtypeid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMEQTYPE> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(EMEQTYPE et) {
        return (!ObjectUtils.isEmpty(et.getEmeqtypeid()))&&(!Objects.isNull(this.getById(et.getEmeqtypeid())));
    }

    @Override
    @Transactional
    public boolean save(EMEQTYPE et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMEQTYPE et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMEQTYPE> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EMEQTYPE> selectByEqtypepid(String emeqtypeid) {
        return baseMapper.selectByEqtypepid(emeqtypeid);
    }

    @Override
    public void removeByEqtypepid(String emeqtypeid) {
        this.remove(new QueryWrapper<EMEQTYPE>().eq("eqtypepid",emeqtypeid));
    }


    /**
     * 查询集合 上级设备类型
     */
    @Override
    public Page<EMEQTYPE> searchPeqType(EMEQTYPESearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMEQTYPE> pages=baseMapper.searchPeqType(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMEQTYPE>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMEQTYPE> searchDefault(EMEQTYPESearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMEQTYPE> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMEQTYPE>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMEQTYPE et){
        //实体关系[DER1N_EMEQTYPE_EMEQTYPE_EQTYPEPID]
        if(!ObjectUtils.isEmpty(et.getEqtypepid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEQTYPE eqtypep=et.getEqtypep();
            if(ObjectUtils.isEmpty(eqtypep)){
                cn.ibizlab.eam.core.eam_core.domain.EMEQTYPE majorEntity=emeqtypeService.get(et.getEqtypepid());
                et.setEqtypep(majorEntity);
                eqtypep=majorEntity;
            }
            et.setEqtypepcode(eqtypep.getEqtypecode());
            et.setEqtypepname(eqtypep.getEqtypeinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


