package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMFILEMENU;
import cn.ibizlab.eam.core.eam_core.filter.EMFILEMENUSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMFILEMENUService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMFILEMENUMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[文件夹目录] 服务对象接口实现
 */
@Slf4j
@Service("EMFILEMENUServiceImpl")
public class EMFILEMENUServiceImpl extends ServiceImpl<EMFILEMENUMapper, EMFILEMENU> implements IEMFILEMENUService {


    private int batchSize = 500;

    @Override
    public boolean checkKey(EMFILEMENU et) {
        return (!ObjectUtils.isEmpty(et.getEmfilemenuid()))&&(!Objects.isNull(this.getById(et.getEmfilemenuid())));
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean save(EMFILEMENU et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMFILEMENU et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMFILEMENU> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EMFILEMENU et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emfilemenuid",et.getEmfilemenuid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmfilemenuid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMFILEMENU> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public EMFILEMENU get(String key) {
        EMFILEMENU et = getById(key);
        if(et==null){
            et=new EMFILEMENU();
            et.setEmfilemenuid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EMFILEMENU getDraft(EMFILEMENU et) {
        return et;
    }

    @Override
    @Transactional
    public boolean create(EMFILEMENU et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmfilemenuid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMFILEMENU> list) {
        saveBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMFILEMENU> searchDefault(EMFILEMENUSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMFILEMENU> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMFILEMENU>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


