package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMITEMPL;
import cn.ibizlab.eam.core.eam_core.filter.EMITEMPLSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMITEMPLService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMITEMPLMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[损溢单] 服务对象接口实现
 */
@Slf4j
@Service("EMITEMPLServiceImpl")
public class EMITEMPLServiceImpl extends ServiceImpl<EMITEMPLMapper, EMITEMPL> implements IEMITEMPLService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMRINService emitemrinService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMService emitemService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSTOREPARTService emstorepartService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSTOREService emstoreService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(EMITEMPL et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emitemplid",et.getEmitemplid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmitemplid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMITEMPL> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(EMITEMPL et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmitemplid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMITEMPL> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMITEMPL et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMITEMPL et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMITEMPL> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(EMITEMPL et) {
        return (!ObjectUtils.isEmpty(et.getEmitemplid()))&&(!Objects.isNull(this.getById(et.getEmitemplid())));
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EMITEMPL get(String key) {
        EMITEMPL et = getById(key);
        if(et==null){
            et=new EMITEMPL();
            et.setEmitemplid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EMITEMPL getDraft(EMITEMPL et) {
        fillParentData(et);
        return et;
    }


	@Override
    public List<EMITEMPL> selectByRid(String emitemrinid) {
        return baseMapper.selectByRid(emitemrinid);
    }

    @Override
    public void removeByRid(String emitemrinid) {
        this.remove(new QueryWrapper<EMITEMPL>().eq("rid",emitemrinid));
    }

	@Override
    public List<EMITEMPL> selectByItemid(String emitemid) {
        return baseMapper.selectByItemid(emitemid);
    }

    @Override
    public void removeByItemid(String emitemid) {
        this.remove(new QueryWrapper<EMITEMPL>().eq("itemid",emitemid));
    }

	@Override
    public List<EMITEMPL> selectByStorepartid(String emstorepartid) {
        return baseMapper.selectByStorepartid(emstorepartid);
    }

    @Override
    public void removeByStorepartid(String emstorepartid) {
        this.remove(new QueryWrapper<EMITEMPL>().eq("storepartid",emstorepartid));
    }

	@Override
    public List<EMITEMPL> selectByStoreid(String emstoreid) {
        return baseMapper.selectByStoreid(emstoreid);
    }

    @Override
    public void removeByStoreid(String emstoreid) {
        this.remove(new QueryWrapper<EMITEMPL>().eq("storeid",emstoreid));
    }

	@Override
    public List<EMITEMPL> selectBySempid(String pfempid) {
        return baseMapper.selectBySempid(pfempid);
    }

    @Override
    public void removeBySempid(String pfempid) {
        this.remove(new QueryWrapper<EMITEMPL>().eq("sempid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMITEMPL> searchDefault(EMITEMPLSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMITEMPL> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMITEMPL>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMITEMPL et){
        //实体关系[DER1N_EMITEMPL_EMITEMRIN_RID]
        if(!ObjectUtils.isEmpty(et.getRid())){
            cn.ibizlab.eam.core.eam_core.domain.EMITEMRIN r=et.getR();
            if(ObjectUtils.isEmpty(r)){
                cn.ibizlab.eam.core.eam_core.domain.EMITEMRIN majorEntity=emitemrinService.get(et.getRid());
                et.setR(majorEntity);
                r=majorEntity;
            }
            et.setRname(r.getItemrininfo());
        }
        //实体关系[DER1N_EMITEMPL_EMITEM_ITEMID]
        if(!ObjectUtils.isEmpty(et.getItemid())){
            cn.ibizlab.eam.core.eam_core.domain.EMITEM item=et.getItem();
            if(ObjectUtils.isEmpty(item)){
                cn.ibizlab.eam.core.eam_core.domain.EMITEM majorEntity=emitemService.get(et.getItemid());
                et.setItem(majorEntity);
                item=majorEntity;
            }
            et.setItemname(item.getIteminfo());
        }
        //实体关系[DER1N_EMITEMPL_EMSTOREPART_STOREPARTID]
        if(!ObjectUtils.isEmpty(et.getStorepartid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSTOREPART storepart=et.getStorepart();
            if(ObjectUtils.isEmpty(storepart)){
                cn.ibizlab.eam.core.eam_core.domain.EMSTOREPART majorEntity=emstorepartService.get(et.getStorepartid());
                et.setStorepart(majorEntity);
                storepart=majorEntity;
            }
            et.setStorepartname(storepart.getEmstorepartname());
        }
        //实体关系[DER1N_EMITEMPL_EMSTORE_STOREID]
        if(!ObjectUtils.isEmpty(et.getStoreid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSTORE store=et.getStore();
            if(ObjectUtils.isEmpty(store)){
                cn.ibizlab.eam.core.eam_core.domain.EMSTORE majorEntity=emstoreService.get(et.getStoreid());
                et.setStore(majorEntity);
                store=majorEntity;
            }
            et.setStorename(store.getStoreinfo());
        }
        //实体关系[DER1N_EMITEMPL_PFEMP_SEMPID]
        if(!ObjectUtils.isEmpty(et.getSempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP semp=et.getSemp();
            if(ObjectUtils.isEmpty(semp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getSempid());
                et.setSemp(majorEntity);
                semp=majorEntity;
            }
            et.setSempname(semp.getEmpinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


