package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMITEMRIN;
import cn.ibizlab.eam.core.eam_core.filter.EMITEMRINSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMITEMRINService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMITEMRINMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[入库单] 服务对象接口实现
 */
@Slf4j
@Service("EMITEMRINServiceImpl")
public class EMITEMRINServiceImpl extends ServiceImpl<EMITEMRINMapper, EMITEMRIN> implements IEMITEMRINService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMCSService emitemcsService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMPLService emitemplService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMROUTService emitemroutService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMService emitemService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPODETAILService empodetailService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSERVICEService emserviceService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSTOREPARTService emstorepartService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSTOREService emstoreService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;

    private int batchSize = 500;

    @Override
    public boolean checkKey(EMITEMRIN et) {
        return (!ObjectUtils.isEmpty(et.getEmitemrinid()))&&(!Objects.isNull(this.getById(et.getEmitemrinid())));
    }

    @Override
    public EMITEMRIN getDraft(EMITEMRIN et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean create(EMITEMRIN et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmitemrinid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMITEMRIN> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EMITEMRIN et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emitemrinid",et.getEmitemrinid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmitemrinid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMITEMRIN> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMITEMRIN et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMITEMRIN et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMITEMRIN> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public EMITEMRIN get(String key) {
        EMITEMRIN et = getById(key);
        if(et==null){
            et=new EMITEMRIN();
            et.setEmitemrinid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }


	@Override
    public List<EMITEMRIN> selectByItemid(String emitemid) {
        return baseMapper.selectByItemid(emitemid);
    }

    @Override
    public void removeByItemid(String emitemid) {
        this.remove(new QueryWrapper<EMITEMRIN>().eq("itemid",emitemid));
    }

	@Override
    public List<EMITEMRIN> selectByPodetailid(String empodetailid) {
        return baseMapper.selectByPodetailid(empodetailid);
    }

    @Override
    public void removeByPodetailid(String empodetailid) {
        this.remove(new QueryWrapper<EMITEMRIN>().eq("podetailid",empodetailid));
    }

	@Override
    public List<EMITEMRIN> selectByEmserviceid(String emserviceid) {
        return baseMapper.selectByEmserviceid(emserviceid);
    }

    @Override
    public void removeByEmserviceid(String emserviceid) {
        this.remove(new QueryWrapper<EMITEMRIN>().eq("emserviceid",emserviceid));
    }

	@Override
    public List<EMITEMRIN> selectByStorepartid(String emstorepartid) {
        return baseMapper.selectByStorepartid(emstorepartid);
    }

    @Override
    public void removeByStorepartid(String emstorepartid) {
        this.remove(new QueryWrapper<EMITEMRIN>().eq("storepartid",emstorepartid));
    }

	@Override
    public List<EMITEMRIN> selectByStoreid(String emstoreid) {
        return baseMapper.selectByStoreid(emstoreid);
    }

    @Override
    public void removeByStoreid(String emstoreid) {
        this.remove(new QueryWrapper<EMITEMRIN>().eq("storeid",emstoreid));
    }

	@Override
    public List<EMITEMRIN> selectByEmpid(String pfempid) {
        return baseMapper.selectByEmpid(pfempid);
    }

    @Override
    public void removeByEmpid(String pfempid) {
        this.remove(new QueryWrapper<EMITEMRIN>().eq("empid",pfempid));
    }

	@Override
    public List<EMITEMRIN> selectBySempid(String pfempid) {
        return baseMapper.selectBySempid(pfempid);
    }

    @Override
    public void removeBySempid(String pfempid) {
        this.remove(new QueryWrapper<EMITEMRIN>().eq("sempid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMITEMRIN> searchDefault(EMITEMRINSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMITEMRIN> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMITEMRIN>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMITEMRIN et){
        //实体关系[DER1N_EMITEMRIN_EMITEM_ITEMID]
        if(!ObjectUtils.isEmpty(et.getItemid())){
            cn.ibizlab.eam.core.eam_core.domain.EMITEM item=et.getItem();
            if(ObjectUtils.isEmpty(item)){
                cn.ibizlab.eam.core.eam_core.domain.EMITEM majorEntity=emitemService.get(et.getItemid());
                et.setItem(majorEntity);
                item=majorEntity;
            }
            et.setItemcode(item.getItemcode());
            et.setUnitid(item.getUnitid());
            et.setUnitname(item.getUnitname());
            et.setItemname(item.getIteminfo());
            et.setItemtypename(item.getItemtypename());
            et.setAvgprice(item.getPrice());
            et.setItembtypeid(item.getItembtypeid());
        }
        //实体关系[DER1N_EMITEMRIN_EMPODETAIL_PODETAILID]
        if(!ObjectUtils.isEmpty(et.getPodetailid())){
            cn.ibizlab.eam.core.eam_core.domain.EMPODETAIL podetail=et.getPodetail();
            if(ObjectUtils.isEmpty(podetail)){
                cn.ibizlab.eam.core.eam_core.domain.EMPODETAIL majorEntity=empodetailService.get(et.getPodetailid());
                et.setPodetail(majorEntity);
                podetail=majorEntity;
            }
            et.setTeamid(podetail.getTeamid());
            et.setLabservicename(podetail.getLabservicename());
            et.setCivo(podetail.getCivo());
            et.setPoid(podetail.getPoid());
            et.setAvgtsfee(podetail.getAvgtsfee());
            et.setWplistid(podetail.getWplistid());
            et.setPodetailname(podetail.getPodetailinfo());
            et.setDeptid(podetail.getDeptid());
            et.setAempid(podetail.getAempid());
            et.setPorempname(podetail.getPorempname());
            et.setShf(podetail.getShf());
            et.setLabserviceid(podetail.getLabserviceid());
        }
        //实体关系[DER1N_EMITEMRIN_EMSERVICE_EMSERVICEID]
        if(!ObjectUtils.isEmpty(et.getEmserviceid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSERVICE emservice=et.getEmservice();
            if(ObjectUtils.isEmpty(emservice)){
                cn.ibizlab.eam.core.eam_core.domain.EMSERVICE majorEntity=emserviceService.get(et.getEmserviceid());
                et.setEmservice(majorEntity);
                emservice=majorEntity;
            }
            et.setEmservicename(emservice.getServiceinfo());
        }
        //实体关系[DER1N_EMITEMRIN_EMSTOREPART_STOREPARTID]
        if(!ObjectUtils.isEmpty(et.getStorepartid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSTOREPART storepart=et.getStorepart();
            if(ObjectUtils.isEmpty(storepart)){
                cn.ibizlab.eam.core.eam_core.domain.EMSTOREPART majorEntity=emstorepartService.get(et.getStorepartid());
                et.setStorepart(majorEntity);
                storepart=majorEntity;
            }
            et.setStorepartname(storepart.getEmstorepartname());
        }
        //实体关系[DER1N_EMITEMRIN_EMSTORE_STOREID]
        if(!ObjectUtils.isEmpty(et.getStoreid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSTORE store=et.getStore();
            if(ObjectUtils.isEmpty(store)){
                cn.ibizlab.eam.core.eam_core.domain.EMSTORE majorEntity=emstoreService.get(et.getStoreid());
                et.setStore(majorEntity);
                store=majorEntity;
            }
            et.setStorename(store.getStoreinfo());
        }
        //实体关系[DER1N_EMITEMRIN_PFEMP_EMPID]
        if(!ObjectUtils.isEmpty(et.getEmpid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP emp=et.getEmp();
            if(ObjectUtils.isEmpty(emp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getEmpid());
                et.setEmp(majorEntity);
                emp=majorEntity;
            }
            et.setEmpname(emp.getEmpinfo());
        }
        //实体关系[DER1N_EMITEMRIN_PFEMP_SEMPID]
        if(!ObjectUtils.isEmpty(et.getSempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP semp=et.getSemp();
            if(ObjectUtils.isEmpty(semp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getSempid());
                et.setSemp(majorEntity);
                semp=majorEntity;
            }
            et.setSempname(semp.getEmpinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


