package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMITEMSUBMAP;
import cn.ibizlab.eam.core.eam_core.filter.EMITEMSUBMAPSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMITEMSUBMAPService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMITEMSUBMAPMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[物品替换件] 服务对象接口实现
 */
@Slf4j
@Service("EMITEMSUBMAPServiceImpl")
public class EMITEMSUBMAPServiceImpl extends ServiceImpl<EMITEMSUBMAPMapper, EMITEMSUBMAP> implements IEMITEMSUBMAPService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMService emitemService;

    private int batchSize = 500;

    @Override
    @Transactional
    public EMITEMSUBMAP get(String key) {
        EMITEMSUBMAP et = getById(key);
        if(et==null){
            et=new EMITEMSUBMAP();
            et.setEmitemsubmapid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean save(EMITEMSUBMAP et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMITEMSUBMAP et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMITEMSUBMAP> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(EMITEMSUBMAP et) {
        return (!ObjectUtils.isEmpty(et.getEmitemsubmapid()))&&(!Objects.isNull(this.getById(et.getEmitemsubmapid())));
    }

    @Override
    @Transactional
    public boolean create(EMITEMSUBMAP et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmitemsubmapid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMITEMSUBMAP> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EMITEMSUBMAP et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emitemsubmapid",et.getEmitemsubmapid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmitemsubmapid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMITEMSUBMAP> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    public EMITEMSUBMAP getDraft(EMITEMSUBMAP et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }


	@Override
    public List<EMITEMSUBMAP> selectByItemid(String emitemid) {
        return baseMapper.selectByItemid(emitemid);
    }

    @Override
    public void removeByItemid(String emitemid) {
        this.remove(new QueryWrapper<EMITEMSUBMAP>().eq("itemid",emitemid));
    }

	@Override
    public List<EMITEMSUBMAP> selectBySubitemid(String emitemid) {
        return baseMapper.selectBySubitemid(emitemid);
    }

    @Override
    public void removeBySubitemid(String emitemid) {
        this.remove(new QueryWrapper<EMITEMSUBMAP>().eq("subitemid",emitemid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMITEMSUBMAP> searchDefault(EMITEMSUBMAPSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMITEMSUBMAP> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMITEMSUBMAP>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMITEMSUBMAP et){
        //实体关系[DER1N_EMITEMSUBMAP_EMITEM_ITEMID]
        if(!ObjectUtils.isEmpty(et.getItemid())){
            cn.ibizlab.eam.core.eam_core.domain.EMITEM item=et.getItem();
            if(ObjectUtils.isEmpty(item)){
                cn.ibizlab.eam.core.eam_core.domain.EMITEM majorEntity=emitemService.get(et.getItemid());
                et.setItem(majorEntity);
                item=majorEntity;
            }
            et.setItemname(item.getIteminfo());
        }
        //实体关系[DER1N_EMITEMSUBMAP_EMITEM_SUBITEMID]
        if(!ObjectUtils.isEmpty(et.getSubitemid())){
            cn.ibizlab.eam.core.eam_core.domain.EMITEM subitem=et.getSubitem();
            if(ObjectUtils.isEmpty(subitem)){
                cn.ibizlab.eam.core.eam_core.domain.EMITEM majorEntity=emitemService.get(et.getSubitemid());
                et.setSubitem(majorEntity);
                subitem=majorEntity;
            }
            et.setSubitemname(subitem.getIteminfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


