package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMPLANTDETAIL;
import cn.ibizlab.eam.core.eam_core.filter.EMPLANTDETAILSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMPLANTDETAILService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMPLANTDETAILMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[计划模板步骤] 服务对象接口实现
 */
@Slf4j
@Service("EMPLANTDETAILServiceImpl")
public class EMPLANTDETAILServiceImpl extends ServiceImpl<EMPLANTDETAILMapper, EMPLANTDETAIL> implements IEMPLANTDETAILService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPLANTEMPLService emplantemplService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(EMPLANTDETAIL et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emplantdetailid",et.getEmplantdetailid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmplantdetailid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMPLANTDETAIL> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    public EMPLANTDETAIL getDraft(EMPLANTDETAIL et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean create(EMPLANTDETAIL et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmplantdetailid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMPLANTDETAIL> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMPLANTDETAIL et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMPLANTDETAIL et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMPLANTDETAIL> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public EMPLANTDETAIL get(String key) {
        EMPLANTDETAIL et = getById(key);
        if(et==null){
            et=new EMPLANTDETAIL();
            et.setEmplantdetailid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public boolean checkKey(EMPLANTDETAIL et) {
        return (!ObjectUtils.isEmpty(et.getEmplantdetailid()))&&(!Objects.isNull(this.getById(et.getEmplantdetailid())));
    }


	@Override
    public List<EMPLANTDETAIL> selectByPlantemplid(String emplantemplid) {
        return baseMapper.selectByPlantemplid(emplantemplid);
    }

    @Override
    public void removeByPlantemplid(String emplantemplid) {
        this.remove(new QueryWrapper<EMPLANTDETAIL>().eq("plantemplid",emplantemplid));
    }

	@Override
    public List<EMPLANTDETAIL> selectByRecvpersonid(String pfempid) {
        return baseMapper.selectByRecvpersonid(pfempid);
    }

    @Override
    public void removeByRecvpersonid(String pfempid) {
        this.remove(new QueryWrapper<EMPLANTDETAIL>().eq("recvpersonid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMPLANTDETAIL> searchDefault(EMPLANTDETAILSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMPLANTDETAIL> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMPLANTDETAIL>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMPLANTDETAIL et){
        //实体关系[DER1N_EMPLANTDETAIL_EMPLANTEMPL_PLANTEMPLID]
        if(!ObjectUtils.isEmpty(et.getPlantemplid())){
            cn.ibizlab.eam.core.eam_core.domain.EMPLANTEMPL plantempl=et.getPlantempl();
            if(ObjectUtils.isEmpty(plantempl)){
                cn.ibizlab.eam.core.eam_core.domain.EMPLANTEMPL majorEntity=emplantemplService.get(et.getPlantemplid());
                et.setPlantempl(majorEntity);
                plantempl=majorEntity;
            }
            et.setPlantemplname(plantempl.getPlantemplinfo());
        }
        //实体关系[DER1N_EMPLANTDETAIL_PFEMP_RECVPERSONID]
        if(!ObjectUtils.isEmpty(et.getRecvpersonid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP recvperson=et.getRecvperson();
            if(ObjectUtils.isEmpty(recvperson)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getRecvpersonid());
                et.setRecvperson(majorEntity);
                recvperson=majorEntity;
            }
            et.setRecvpersonname(recvperson.getEmpinfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


