package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMPLANTEMPL;
import cn.ibizlab.eam.core.eam_core.filter.EMPLANTEMPLSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMPLANTEMPLService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMPLANTEMPLMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[计划模板] 服务对象接口实现
 */
@Slf4j
@Service("EMPLANTEMPLServiceImpl")
public class EMPLANTEMPLServiceImpl extends ServiceImpl<EMPLANTEMPLMapper, EMPLANTEMPL> implements IEMPLANTEMPLService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPLANTDETAILService emplantdetailService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPLANService emplanService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMACCLASSService emacclassService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSERVICEService emserviceService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFDEPTService pfdeptService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFTEAMService pfteamService;

    private int batchSize = 500;

    @Override
    @Transactional
    public EMPLANTEMPL get(String key) {
        EMPLANTEMPL et = getById(key);
        if(et==null){
            et=new EMPLANTEMPL();
            et.setEmplantemplid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EMPLANTEMPL getDraft(EMPLANTEMPL et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean create(EMPLANTEMPL et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmplantemplid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMPLANTEMPL> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public boolean checkKey(EMPLANTEMPL et) {
        return (!ObjectUtils.isEmpty(et.getEmplantemplid()))&&(!Objects.isNull(this.getById(et.getEmplantemplid())));
    }

    @Override
    @Transactional
    public boolean update(EMPLANTEMPL et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emplantemplid",et.getEmplantemplid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmplantemplid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMPLANTEMPL> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMPLANTEMPL et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMPLANTEMPL et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMPLANTEMPL> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EMPLANTEMPL> selectByAcclassid(String emacclassid) {
        return baseMapper.selectByAcclassid(emacclassid);
    }

    @Override
    public void removeByAcclassid(String emacclassid) {
        this.remove(new QueryWrapper<EMPLANTEMPL>().eq("acclassid",emacclassid));
    }

	@Override
    public List<EMPLANTEMPL> selectByRserviceid(String emserviceid) {
        return baseMapper.selectByRserviceid(emserviceid);
    }

    @Override
    public void removeByRserviceid(String emserviceid) {
        this.remove(new QueryWrapper<EMPLANTEMPL>().eq("rserviceid",emserviceid));
    }

	@Override
    public List<EMPLANTEMPL> selectByRdeptid(String pfdeptid) {
        return baseMapper.selectByRdeptid(pfdeptid);
    }

    @Override
    public void removeByRdeptid(String pfdeptid) {
        this.remove(new QueryWrapper<EMPLANTEMPL>().eq("rdeptid",pfdeptid));
    }

	@Override
    public List<EMPLANTEMPL> selectByMpersonid(String pfempid) {
        return baseMapper.selectByMpersonid(pfempid);
    }

    @Override
    public void removeByMpersonid(String pfempid) {
        this.remove(new QueryWrapper<EMPLANTEMPL>().eq("mpersonid",pfempid));
    }

	@Override
    public List<EMPLANTEMPL> selectByRecvpersonid(String pfempid) {
        return baseMapper.selectByRecvpersonid(pfempid);
    }

    @Override
    public void removeByRecvpersonid(String pfempid) {
        this.remove(new QueryWrapper<EMPLANTEMPL>().eq("recvpersonid",pfempid));
    }

	@Override
    public List<EMPLANTEMPL> selectByRempid(String pfempid) {
        return baseMapper.selectByRempid(pfempid);
    }

    @Override
    public void removeByRempid(String pfempid) {
        this.remove(new QueryWrapper<EMPLANTEMPL>().eq("rempid",pfempid));
    }

	@Override
    public List<EMPLANTEMPL> selectByRteamid(String pfteamid) {
        return baseMapper.selectByRteamid(pfteamid);
    }

    @Override
    public void removeByRteamid(String pfteamid) {
        this.remove(new QueryWrapper<EMPLANTEMPL>().eq("rteamid",pfteamid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMPLANTEMPL> searchDefault(EMPLANTEMPLSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMPLANTEMPL> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMPLANTEMPL>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMPLANTEMPL et){
        //实体关系[DER1N_EMPLANTEMPL_EMACCLASS_ACCLASSID]
        if(!ObjectUtils.isEmpty(et.getAcclassid())){
            cn.ibizlab.eam.core.eam_core.domain.EMACCLASS acclass=et.getAcclass();
            if(ObjectUtils.isEmpty(acclass)){
                cn.ibizlab.eam.core.eam_core.domain.EMACCLASS majorEntity=emacclassService.get(et.getAcclassid());
                et.setAcclass(majorEntity);
                acclass=majorEntity;
            }
            et.setAcclassname(acclass.getEmacclassname());
        }
        //实体关系[DER1N_EMPLANTEMPL_EMSERVICE_RSERVICEID]
        if(!ObjectUtils.isEmpty(et.getRserviceid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSERVICE rservice=et.getRservice();
            if(ObjectUtils.isEmpty(rservice)){
                cn.ibizlab.eam.core.eam_core.domain.EMSERVICE majorEntity=emserviceService.get(et.getRserviceid());
                et.setRservice(majorEntity);
                rservice=majorEntity;
            }
            et.setRservicename(rservice.getServiceinfo());
        }
        //实体关系[DER1N_EMPLANTEMPL_PFDEPT_RDEPTID]
        if(!ObjectUtils.isEmpty(et.getRdeptid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFDEPT rdept=et.getRdept();
            if(ObjectUtils.isEmpty(rdept)){
                cn.ibizlab.eam.core.eam_pf.domain.PFDEPT majorEntity=pfdeptService.get(et.getRdeptid());
                et.setRdept(majorEntity);
                rdept=majorEntity;
            }
            et.setRdeptname(rdept.getDeptinfo());
        }
        //实体关系[DER1N_EMPLANTEMPL_PFEMP_MPERSONID]
        if(!ObjectUtils.isEmpty(et.getMpersonid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP mperson=et.getMperson();
            if(ObjectUtils.isEmpty(mperson)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getMpersonid());
                et.setMperson(majorEntity);
                mperson=majorEntity;
            }
            et.setMpersonname(mperson.getEmpinfo());
        }
        //实体关系[DER1N_EMPLANTEMPL_PFEMP_RECVPERSONID]
        if(!ObjectUtils.isEmpty(et.getRecvpersonid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP recvperson=et.getRecvperson();
            if(ObjectUtils.isEmpty(recvperson)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getRecvpersonid());
                et.setRecvperson(majorEntity);
                recvperson=majorEntity;
            }
            et.setRecvpersonname(recvperson.getEmpinfo());
        }
        //实体关系[DER1N_EMPLANTEMPL_PFEMP_REMPID]
        if(!ObjectUtils.isEmpty(et.getRempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP remp=et.getRemp();
            if(ObjectUtils.isEmpty(remp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getRempid());
                et.setRemp(majorEntity);
                remp=majorEntity;
            }
            et.setRempname(remp.getEmpinfo());
        }
        //实体关系[DER1N_EMPLANTEMPL_PFTEAM_RTEAMID]
        if(!ObjectUtils.isEmpty(et.getRteamid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFTEAM rteam=et.getRteam();
            if(ObjectUtils.isEmpty(rteam)){
                cn.ibizlab.eam.core.eam_pf.domain.PFTEAM majorEntity=pfteamService.get(et.getRteamid());
                et.setRteam(majorEntity);
                rteam=majorEntity;
            }
            et.setRteamname(rteam.getTeaminfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


