package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMRESEMP;
import cn.ibizlab.eam.core.eam_core.filter.EMRESEMPSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMRESEMPService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMRESEMPMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[员工资源] 服务对象接口实现
 */
@Slf4j
@Service("EMRESEMPServiceImpl")
public class EMRESEMPServiceImpl extends ServiceImpl<EMRESEMPMapper, EMRESEMP> implements IEMRESEMPService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQUIPService emequipService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMRESREFOBJService emresrefobjService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;

    private int batchSize = 500;

    @Override
    public EMRESEMP getDraft(EMRESEMP et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public EMRESEMP get(String key) {
        EMRESEMP et = getById(key);
        if(et==null){
            et=new EMRESEMP();
            et.setEmresempid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean create(EMRESEMP et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmresempid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMRESEMP> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EMRESEMP et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emresempid",et.getEmresempid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmresempid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMRESEMP> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMRESEMP et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMRESEMP et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMRESEMP> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public boolean checkKey(EMRESEMP et) {
        return (!ObjectUtils.isEmpty(et.getEmresempid()))&&(!Objects.isNull(this.getById(et.getEmresempid())));
    }


	@Override
    public List<EMRESEMP> selectByEquipid(String emequipid) {
        return baseMapper.selectByEquipid(emequipid);
    }

    @Override
    public void removeByEquipid(String emequipid) {
        this.remove(new QueryWrapper<EMRESEMP>().eq("equipid",emequipid));
    }

	@Override
    public List<EMRESEMP> selectByResrefobjid(String emresrefobjid) {
        return baseMapper.selectByResrefobjid(emresrefobjid);
    }

    @Override
    public void removeByResrefobjid(String emresrefobjid) {
        this.remove(new QueryWrapper<EMRESEMP>().eq("resrefobjid",emresrefobjid));
    }

	@Override
    public List<EMRESEMP> selectByResid(String pfempid) {
        return baseMapper.selectByResid(pfempid);
    }

    @Override
    public void removeByResid(String pfempid) {
        this.remove(new QueryWrapper<EMRESEMP>().eq("resid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMRESEMP> searchDefault(EMRESEMPSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMRESEMP> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMRESEMP>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMRESEMP et){
        //实体关系[DER1N_EMRESEMP_EMEQUIP_EQUIPID]
        if(!ObjectUtils.isEmpty(et.getEquipid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEQUIP equip=et.getEquip();
            if(ObjectUtils.isEmpty(equip)){
                cn.ibizlab.eam.core.eam_core.domain.EMEQUIP majorEntity=emequipService.get(et.getEquipid());
                et.setEquip(majorEntity);
                equip=majorEntity;
            }
            et.setEquipname(equip.getEquipinfo());
        }
        //实体关系[DER1N_EMRESEMP_EMRESREFOBJ_RESREFOBJID]
        if(!ObjectUtils.isEmpty(et.getResrefobjid())){
            cn.ibizlab.eam.core.eam_core.domain.EMRESREFOBJ resrefobj=et.getResrefobj();
            if(ObjectUtils.isEmpty(resrefobj)){
                cn.ibizlab.eam.core.eam_core.domain.EMRESREFOBJ majorEntity=emresrefobjService.get(et.getResrefobjid());
                et.setResrefobj(majorEntity);
                resrefobj=majorEntity;
            }
            et.setResrefobjname(resrefobj.getResrefobjinfo());
        }
        //实体关系[DER1N_EMRESEMP_PFEMP_RESID]
        if(!ObjectUtils.isEmpty(et.getResid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP res=et.getRes();
            if(ObjectUtils.isEmpty(res)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getResid());
                et.setRes(majorEntity);
                res=majorEntity;
            }
            et.setResname(res.getEmpinfo());
            et.setEmpcode(res.getEmpcode());
            et.setTeamname(res.getMajorteamname());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


