package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMWPLIST;
import cn.ibizlab.eam.core.eam_core.filter.EMWPLISTSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMWPLISTService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMWPLISTMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[采购申请] 服务对象接口实现
 */
@Slf4j
@Service("EMWPLISTServiceImpl")
public class EMWPLISTServiceImpl extends ServiceImpl<EMWPLISTMapper, EMWPLIST> implements IEMWPLISTService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPODETAILService empodetailService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEQUIPService emequipService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMITEMService emitemService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMOBJECTService emobjectService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMSERVICEService emserviceService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMWPLISTCOSTService emwplistcostService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFDEPTService pfdeptService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEMPService pfempService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFTEAMService pfteamService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EMWPLIST et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmwplistid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMWPLIST> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public EMWPLIST get(String key) {
        EMWPLIST et = getById(key);
        if(et==null){
            et=new EMWPLIST();
            et.setEmwplistid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EMWPLIST getDraft(EMWPLIST et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean save(EMWPLIST et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMWPLIST et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<EMWPLIST> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(EMWPLIST et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emwplistid",et.getEmwplistid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmwplistid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMWPLIST> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    public boolean checkKey(EMWPLIST et) {
        return (!ObjectUtils.isEmpty(et.getEmwplistid()))&&(!Objects.isNull(this.getById(et.getEmwplistid())));
    }


	@Override
    public List<EMWPLIST> selectByEquipid(String emequipid) {
        return baseMapper.selectByEquipid(emequipid);
    }

    @Override
    public void removeByEquipid(String emequipid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("equipid",emequipid));
    }

	@Override
    public List<EMWPLIST> selectByItemid(String emitemid) {
        return baseMapper.selectByItemid(emitemid);
    }

    @Override
    public void removeByItemid(String emitemid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("itemid",emitemid));
    }

	@Override
    public List<EMWPLIST> selectByObjid(String emobjectid) {
        return baseMapper.selectByObjid(emobjectid);
    }

    @Override
    public void removeByObjid(String emobjectid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("objid",emobjectid));
    }

	@Override
    public List<EMWPLIST> selectByEmserviceid(String emserviceid) {
        return baseMapper.selectByEmserviceid(emserviceid);
    }

    @Override
    public void removeByEmserviceid(String emserviceid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("emserviceid",emserviceid));
    }

	@Override
    public List<EMWPLIST> selectByWplistcostid(String emwplistcostid) {
        return baseMapper.selectByWplistcostid(emwplistcostid);
    }

    @Override
    public void removeByWplistcostid(String emwplistcostid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("wplistcostid",emwplistcostid));
    }

	@Override
    public List<EMWPLIST> selectByDeptid(String pfdeptid) {
        return baseMapper.selectByDeptid(pfdeptid);
    }

    @Override
    public void removeByDeptid(String pfdeptid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("deptid",pfdeptid));
    }

	@Override
    public List<EMWPLIST> selectByAempid(String pfempid) {
        return baseMapper.selectByAempid(pfempid);
    }

    @Override
    public void removeByAempid(String pfempid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("aempid",pfempid));
    }

	@Override
    public List<EMWPLIST> selectByApprempid(String pfempid) {
        return baseMapper.selectByApprempid(pfempid);
    }

    @Override
    public void removeByApprempid(String pfempid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("apprempid",pfempid));
    }

	@Override
    public List<EMWPLIST> selectByRempid(String pfempid) {
        return baseMapper.selectByRempid(pfempid);
    }

    @Override
    public void removeByRempid(String pfempid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("rempid",pfempid));
    }

	@Override
    public List<EMWPLIST> selectByTeamid(String pfteamid) {
        return baseMapper.selectByTeamid(pfteamid);
    }

    @Override
    public void removeByTeamid(String pfteamid) {
        this.remove(new QueryWrapper<EMWPLIST>().eq("teamid",pfteamid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMWPLIST> searchDefault(EMWPLISTSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMWPLIST> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMWPLIST>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 已到货采购申请
     */
    @Override
    public Page<EMWPLIST> searchMain6_8692(EMWPLISTSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMWPLIST> pages=baseMapper.searchMain6_8692(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMWPLIST>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMWPLIST et){
        //实体关系[DER1N_EMWPLIST_EMEQUIP_EQUIPID]
        if(!ObjectUtils.isEmpty(et.getEquipid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEQUIP equip=et.getEquip();
            if(ObjectUtils.isEmpty(equip)){
                cn.ibizlab.eam.core.eam_core.domain.EMEQUIP majorEntity=emequipService.get(et.getEquipid());
                et.setEquip(majorEntity);
                equip=majorEntity;
            }
            et.setEquipname(equip.getEquipinfo());
        }
        //实体关系[DER1N_EMWPLIST_EMITEM_ITEMID]
        if(!ObjectUtils.isEmpty(et.getItemid())){
            cn.ibizlab.eam.core.eam_core.domain.EMITEM item=et.getItem();
            if(ObjectUtils.isEmpty(item)){
                cn.ibizlab.eam.core.eam_core.domain.EMITEM majorEntity=emitemService.get(et.getItemid());
                et.setItem(majorEntity);
                item=majorEntity;
            }
            et.setNo3q(item.getNo3q());
            et.setItemcode(item.getItemcode());
            et.setUnitname(item.getUnitname());
            et.setItemname(item.getIteminfo());
            et.setItembtypeid(item.getItembtypeid());
            et.setItemnameShow(item.getEmitemname());
            et.setAvgprice(item.getPrice());
            et.setUnitid(item.getUnitid());
        }
        //实体关系[DER1N_EMWPLIST_EMOBJECT_OBJID]
        if(!ObjectUtils.isEmpty(et.getObjid())){
            cn.ibizlab.eam.core.eam_core.domain.EMOBJECT obj=et.getObj();
            if(ObjectUtils.isEmpty(obj)){
                cn.ibizlab.eam.core.eam_core.domain.EMOBJECT majorEntity=emobjectService.get(et.getObjid());
                et.setObj(majorEntity);
                obj=majorEntity;
            }
            et.setObjname(obj.getObjectinfo());
        }
        //实体关系[DER1N_EMWPLIST_EMSERVICE_EMSERVICEID]
        if(!ObjectUtils.isEmpty(et.getEmserviceid())){
            cn.ibizlab.eam.core.eam_core.domain.EMSERVICE emservice=et.getEmservice();
            if(ObjectUtils.isEmpty(emservice)){
                cn.ibizlab.eam.core.eam_core.domain.EMSERVICE majorEntity=emserviceService.get(et.getEmserviceid());
                et.setEmservice(majorEntity);
                emservice=majorEntity;
            }
            et.setEmservicename(emservice.getServiceinfo());
        }
        //实体关系[DER1N_EMWPLIST_EMWPLISTCOST_WPLISTCOSTID]
        if(!ObjectUtils.isEmpty(et.getWplistcostid())){
            cn.ibizlab.eam.core.eam_core.domain.EMWPLISTCOST wplistcost=et.getWplistcost();
            if(ObjectUtils.isEmpty(wplistcost)){
                cn.ibizlab.eam.core.eam_core.domain.EMWPLISTCOST majorEntity=emwplistcostService.get(et.getWplistcostid());
                et.setWplistcost(majorEntity);
                wplistcost=majorEntity;
            }
            et.setLabservicename(wplistcost.getLabservicename());
            et.setWplistcostname(wplistcost.getWplistcostinfo());
            et.setLabserviceid(wplistcost.getLabserviceid());
        }
        //实体关系[DER1N_EMWPLIST_PFDEPT_DEPTID]
        if(!ObjectUtils.isEmpty(et.getDeptid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFDEPT dept=et.getDept();
            if(ObjectUtils.isEmpty(dept)){
                cn.ibizlab.eam.core.eam_pf.domain.PFDEPT majorEntity=pfdeptService.get(et.getDeptid());
                et.setDept(majorEntity);
                dept=majorEntity;
            }
            et.setDeptname(dept.getDeptinfo());
            et.setSdept(dept.getSdept());
        }
        //实体关系[DER1N_EMWPLIST_PFEMP_AEMPID]
        if(!ObjectUtils.isEmpty(et.getAempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP aemp=et.getAemp();
            if(ObjectUtils.isEmpty(aemp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getAempid());
                et.setAemp(majorEntity);
                aemp=majorEntity;
            }
            et.setAempname(aemp.getEmpinfo());
        }
        //实体关系[DER1N_EMWPLIST_PFEMP_APPREMPID]
        if(!ObjectUtils.isEmpty(et.getApprempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP appremp=et.getAppremp();
            if(ObjectUtils.isEmpty(appremp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getApprempid());
                et.setAppremp(majorEntity);
                appremp=majorEntity;
            }
            et.setApprempname(appremp.getEmpinfo());
        }
        //实体关系[DER1N_EMWPLIST_PFEMP_REMPID]
        if(!ObjectUtils.isEmpty(et.getRempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEMP remp=et.getRemp();
            if(ObjectUtils.isEmpty(remp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEMP majorEntity=pfempService.get(et.getRempid());
                et.setRemp(majorEntity);
                remp=majorEntity;
            }
            et.setRempname(remp.getEmpinfo());
        }
        //实体关系[DER1N_EMWPLIST_PFTEAM_TEAMID]
        if(!ObjectUtils.isEmpty(et.getTeamid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFTEAM team=et.getTeam();
            if(ObjectUtils.isEmpty(team)){
                cn.ibizlab.eam.core.eam_pf.domain.PFTEAM majorEntity=pfteamService.get(et.getTeamid());
                et.setTeam(majorEntity);
                team=majorEntity;
            }
            et.setTeamname(team.getTeaminfo());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


